import { Interaction, SlashCommandBuilder } from 'discord.js';
import { saveSubscriptions, subscriptions, timeAnnouncements } from '../lib/subscriptions';

module.exports = {
  data: new SlashCommandBuilder()
    .setName('subscribe')
    .setDescription('Subscribe/unsubscribe to a time announcement')
    .addStringOption(option => 
      option
        .setName('type')
        .setChoices(...Object.keys(timeAnnouncements).map(l => ({name: l, value: l})))
        .setDescription('The name of the time announcement')
    )
    .setDefaultMemberPermissions('0'),

  execute: async (interaction: Interaction) => {
    if (!interaction.isChatInputCommand()) return;
    
    await interaction.deferReply({ephemeral: true});

    const announcementType = interaction.options.getString('type', true);

    if (subscriptions[announcementType] && subscriptions[announcementType].includes(interaction.channelId)) {
      subscriptions[announcementType] = subscriptions[announcementType].filter(id => id !== interaction.channelId);
      await interaction.followUp({
        content: `<#${interaction.channelId}> has been unsubscribed from \`${announcementType}\``
      });
    } else {
      subscriptions[announcementType] = subscriptions[announcementType] || [];
      subscriptions[announcementType].push(interaction.channelId);
      await interaction.followUp({
        content: `<#${interaction.channelId}> has been subscribed to \`${announcementType}\``
      });
    }
    saveSubscriptions();
  }
};